@@ -1,4 +1,4 @@
-ST.CARadaptive <- function(formula, family, data=NULL,  trials=NULL, W, burnin, n.sample, thin=1, prior.mean.beta=NULL, prior.var.beta=NULL, prior.nu2=NULL, prior.tau2=NULL,  rhofix = NULL, epsilon = 0, verbose=TRUE)
+ST.CARadaptive <- function(formula, family, data=NULL,  trials=NULL, W, burnin, n.sample, thin=1, prior.mean.beta=NULL, prior.var.beta=NULL, prior.nu2=NULL, prior.tau2=NULL,  rhofix = NULL, epsilon = 0, MALA=TRUE, verbose=TRUE)
 {
     ## This is a wrapper function for the following three functions.
     ## binomial.CARadaptive
@@ -10,7 +10,7 @@ ST.CARadaptive <- function(formula, family, data=NULL,  trials=NULL, W, burnin,
     if(family=="binomial")
     {
         if(is.null(trials)) stop("a binomial model was specified but the trials arugment was not specified", call.=FALSE)
-        model <- binomial.CARadaptive(formula=formula, data=data,  trials=trials, W=W, burnin=burnin, n.sample=n.sample, thin=thin, prior.mean.beta=prior.mean.beta, prior.var.beta=prior.var.beta, prior.tau2=prior.tau2, rhofix = rhofix, epsilon = epsilon, verbose=verbose)
+        model <- binomial.CARadaptive(formula=formula, data=data,  trials=trials, W=W, burnin=burnin, n.sample=n.sample, thin=thin, prior.mean.beta=prior.mean.beta, prior.var.beta=prior.var.beta, prior.tau2=prior.tau2, rhofix = rhofix, epsilon = epsilon, MALA=MALA, verbose=verbose)
     }else if(family=="gaussian")
     {
         if(!is.null(trials)) stop("you do not need a trials arugment as a binomial model was not specified", call.=FALSE)
@@ -18,7 +18,7 @@ ST.CARadaptive <- function(formula, family, data=NULL,  trials=NULL, W, burnin,
     }else if(family=="poisson")
     {
         if(!is.null(trials)) stop("you do not need a trials arugment as a binomial model was not specified", call.=FALSE)
-        model <- poisson.CARadaptive(formula=formula, data=data,  W=W, burnin=burnin, n.sample=n.sample, thin=thin, prior.mean.beta=prior.mean.beta, prior.var.beta=prior.var.beta, prior.tau2=prior.tau2, rhofix = rhofix, epsilon = epsilon, verbose=verbose)          
+        model <- poisson.CARadaptive(formula=formula, data=data,  W=W, burnin=burnin, n.sample=n.sample, thin=thin, prior.mean.beta=prior.mean.beta, prior.var.beta=prior.var.beta, prior.tau2=prior.tau2, rhofix = rhofix, epsilon = epsilon, MALA=MALA, verbose=verbose)          
     }else
     {
         stop("the family arugment is not one of `binomial', `gaussian' or `poisson'.", call.=FALSE)     
