@@ -1,115 +1,97 @@
 gaussian.CARlinear <- function(formula, data=NULL, W, burnin, n.sample, thin=1,  prior.mean.beta=NULL, prior.var.beta=NULL, prior.mean.alpha=NULL, prior.var.alpha=NULL, prior.nu2=NULL, prior.tau2=NULL, fix.rho.slo=FALSE, rho.slo=NULL, fix.rho.int=FALSE, rho.int=NULL,  verbose=TRUE)
 {
-#### Check on the verbose option
-    if(is.null(verbose)) verbose=TRUE     
-    if(!is.logical(verbose)) stop("the verbose option is not logical.", call.=FALSE)
-    
-    if(verbose)
-    {
-        cat("Setting up the model\n")
-        a<-proc.time()
-    }else{}
-    
-    
-    
 ##############################################
 #### Format the arguments and check for errors
 ##############################################
-#### Overall formula object
-frame <- try(suppressWarnings(model.frame(formula, data=data, na.action=na.pass)), silent=TRUE)
-    if(class(frame)=="try-error") stop("the formula inputted contains an error, e.g the variables may be different lengths or the data object has not been specified.", call.=FALSE)
+#### Verbose
+a <- common.verbose(verbose)  
     
     
-#### Design matrix
-## Create the matrix
-X <- try(suppressWarnings(model.matrix(object=attr(frame, "terms"), data=frame)), silent=TRUE)
-    if(class(X)=="try-error") stop("the covariate matrix contains inappropriate values.", call.=FALSE)
-    if(sum(is.na(X))>0) stop("the covariate matrix contains missing 'NA' values.", call.=FALSE)
-N.all <- nrow(X)
-p <- ncol(X)
+#### Frame object
+frame.results <- common.frame(formula, data, "gaussian")
+N.all <- frame.results$n
+p <- frame.results$p
+X <- frame.results$X
+X.standardised <- frame.results$X.standardised
+X.sd <- frame.results$X.sd
+X.mean <- frame.results$X.mean
+X.indicator <- frame.results$X.indicator 
+offset <- frame.results$offset
+Y <- frame.results$Y
+Y.miss <- frame.results$Y.miss
+which.miss <- frame.results$which.miss
+n.miss <- frame.results$n.miss  
+X.short <- X.standardised[which.miss==1, ]  
     
     
-    
-## Check for linearly related columns
-cor.X <- suppressWarnings(cor(X))
-diag(cor.X) <- 0
-    
-    if(max(cor.X, na.rm=TRUE)==1) stop("the covariate matrix has two exactly linearly related columns.", call.=FALSE)
-    if(min(cor.X, na.rm=TRUE)==-1) stop("the covariate matrix has two exactly linearly related columns.", call.=FALSE)
-    
-    if(p>1)
+#### Check on the rho arguments
+    if(!is.logical(fix.rho.int)) stop("fix.rho.int is not logical.", call.=FALSE)   
+    if(fix.rho.int & is.null(rho.int)) stop("rho.int is fixed but an initial value was not set.", call.=FALSE)   
+    if(fix.rho.int & !is.numeric(rho.int) ) stop("rho.S is not numeric.", call.=FALSE)  
+    if(!is.logical(fix.rho.slo)) stop("fix.rho.slo is not logical.", call.=FALSE)   
+    if(fix.rho.slo & is.null(rho.slo)) stop("rho.slo is fixed but an initial value was not set.", call.=FALSE)   
+    if(fix.rho.slo & !is.numeric(rho.slo) ) stop("rho.slo is not numeric.", call.=FALSE)  
+    if(fix.rho.int)
     {
-        if(sort(apply(X, 2, sd))[2]==0) stop("the covariate matrix has two intercept terms.", call.=FALSE)
+    rho <- rho.int
     }else
     {
+    rho <- runif(1)       
     }
-    
-## Standardise the matrix
-X.standardised <- X
-X.sd <- apply(X, 2, sd)
-X.mean <- apply(X, 2, mean)
-X.indicator <- rep(NA, p)       # To determine which parameter estimates to transform back
-    
-    for(j in 1:p)
+    if(fix.rho.slo)
     {
-        if(length(table(X[ ,j]))>2)
-        {
-            X.indicator[j] <- 1
-            X.standardised[ ,j] <- (X[ ,j] - mean(X[ ,j])) / sd(X[ ,j])
-        }else if(length(table(X[ ,j]))==1)
-        {
-            X.indicator[j] <- 2
-        }else
-        {
-            X.indicator[j] <- 0
-        }
-    }
+    lambda <- rho.slo
+    }else
+    {
+    lambda <- runif(1)       
+    }   
+    if(rho<0 ) stop("rho.int is outside the range [0, 1].", call.=FALSE)  
+    if(rho>1 ) stop("rho.int is outside the range [0, 1].", call.=FALSE)  
+    if(lambda<0 ) stop("rho.slo is outside the range [0, 1].", call.=FALSE)  
+    if(lambda>1 ) stop("rho.slo is outside the range [0, 1].", call.=FALSE)  
+
     
-    
-    
-#### Format and check the neighbourhood matrix W
-if(!is.matrix(W)) stop("W is not a matrix.", call.=FALSE)
-K <- nrow(W)
-if(ncol(W)!= K) stop("W has the wrong number of columns.", call.=FALSE)
-if(nrow(W)!= K) stop("W has the wrong number of rows.", call.=FALSE)
-if(floor(N.all/K)!=ceiling(N.all/K)) stop("The number of spatial areas is not a multiple of the number of data points.", call.=FALSE)
+#### CAR quantities
+rho.sum <- rho + lambda
+fix.rho.sum <- as.logical(fix.rho.int * fix.rho.slo)
+W.quants <- common.Wcheckformat.leroux(W, fix.rho.sum, rho.sum)
+K <- W.quants$n
 N <- N.all / K
-if(sum(is.na(W))>0) stop("W has missing 'NA' values.", call.=FALSE)
-if(!is.numeric(W)) stop("W has non-numeric values.", call.=FALSE)
-if(min(W)<0) stop("W has negative elements.", call.=FALSE)
-if(!is.symmetric.matrix(W)) stop("W is not symmetric.", call.=FALSE)
+W <- W.quants$W
+W.triplet <- W.quants$W.triplet
+W.n.triplet <- W.quants$n.triplet
+W.triplet.sum <- W.quants$W.triplet.sum
+n.neighbours <- W.quants$n.neighbours 
+W.begfin <- W.quants$W.begfin
 
-
-
-#### Response variable
-Y <- model.response(frame)
-which.miss <- as.numeric(!is.na(Y))
-which.miss.mat <- matrix(which.miss, nrow=K, ncol=N, byrow=FALSE)
-n.miss <- N.all - sum(which.miss)
-Y.miss <- Y
-Y.miss[which.miss==0] <- median(Y, na.rm=TRUE)    
-if(!is.numeric(Y)) stop("the response variable has non-numeric values.", call.=FALSE)
-X.short <- X.standardised[which.miss==1, ]    
-
-       
-#### Offset variable
-offset <- try(model.offset(frame), silent=TRUE)
-    if(class(offset)=="try-error")   stop("the offset is not numeric.", call.=FALSE)
-    if(is.null(offset))  offset <- rep(0,N.all)
-    if(sum(is.na(offset))>0) stop("the offset has missing 'NA' values.", call.=FALSE)
-    if(!is.numeric(offset)) stop("the offset variable has non-numeric values.", call.=FALSE)
     
+#### Priors
+    if(is.null(prior.mean.beta)) prior.mean.beta <- rep(0, p)
+    if(is.null(prior.var.beta)) prior.var.beta <- rep(1000, p)
+    if(is.null(prior.tau2)) prior.tau2 <- c(1, 0.01)
+    if(is.null(prior.nu2)) prior.nu2 <- c(1, 0.01)
+    if(is.null(prior.mean.alpha)) prior.mean.alpha <- rep(0, 1)
+    if(is.null(prior.var.alpha)) prior.var.alpha <- rep(1000, 1)
+prior.beta.check(prior.mean.beta, prior.var.beta, p)
+prior.var.check(prior.tau2)
+prior.var.check(prior.nu2)
+    if(length(prior.mean.alpha)!=1) stop("the prior mean for alpha is the wrong length.", call.=FALSE)    
+    if(!is.numeric(prior.mean.alpha)) stop("the  prior mean for alpha is not numeric.", call.=FALSE)    
+    if(sum(is.na(prior.mean.alpha))!=0) stop("the prior mean for alpha has missing values.", call.=FALSE)       
+    if(length(prior.var.alpha)!=1) stop("the prior variance for alpha is the wrong length.", call.=FALSE)    
+    if(!is.numeric(prior.var.alpha)) stop("the  prior variance for alpha is not numeric.", call.=FALSE)    
+    if(sum(is.na(prior.var.alpha))!=0) stop("the  prior variance for alpha has missing values.", call.=FALSE)    
+    if(min(prior.var.alpha) <=0) stop("the prior variance for alpha has elements less than zero", call.=FALSE)
+
     
-## Check for errors on rho and fix.rho
-if(!is.logical(fix.rho.int)) stop("fix.rho.int is not logical.", call.=FALSE)   
-if(fix.rho.int & is.null(rho.int)) stop("rho.int is fixed but an initial value was not set.", call.=FALSE)   
-if(fix.rho.int & !is.numeric(rho.int) ) stop("rho.S is not numeric.", call.=FALSE)  
-if(!is.logical(fix.rho.slo)) stop("fix.rho.slo is not logical.", call.=FALSE)   
-if(fix.rho.slo & is.null(rho.slo)) stop("rho.slo is fixed but an initial value was not set.", call.=FALSE)   
-if(fix.rho.slo & !is.numeric(rho.slo) ) stop("rho.slo is not numeric.", call.=FALSE)      
-  
-    
-#### Specify the initial parameter values
+#### MCMC quantities - burnin, n.sample, thin
+common.burnin.nsample.thin.check(burnin, n.sample, thin)
+
+
+
+#############################
+#### Initial parameter values
+#############################
 time <-(1:N - mean(1:N))/N
 time.all <- kronecker(time, rep(1,K))
 mod.glm <- glm(Y~X.standardised-1 + time.all, offset=offset)
@@ -126,113 +108,28 @@ delta <- rnorm(n=K, mean=0, sd = res.sd)
 tau2.phi <- var(phi)/10
 tau2.delta <- var(delta)/10
 
-if(fix.rho.int)
-{
-    rho <- rho.int
-}else
-{
-    rho <- runif(1)       
-}
+ 
 
-if(fix.rho.slo)
-{
-    lambda <- rho.slo
-}else
-{
-    lambda <- runif(1)       
-}   
-
-if(rho<0 ) stop("rho.int is outside the range [0, 1].", call.=FALSE)  
-if(rho>1 ) stop("rho.int is outside the range [0, 1].", call.=FALSE)  
-if(lambda<0 ) stop("rho.slo is outside the range [0, 1].", call.=FALSE)  
-if(lambda>1 ) stop("rho.slo is outside the range [0, 1].", call.=FALSE)  
-    
-if(fix.rho.int & rho==0 & fix.rho.slo & lambda==0)
-{
-    ## Set up a dummy W matrix to use in the code as it will not affect the results
-    W <- array(0, c(K,K))
-    for(r in 2:K)
-    {
-        W[(r-1), r] <- 1   
-        W[r, (r-1)] <- 1
-    }
-}else
-{
-    if(min(apply(W, 1, sum))==0) stop("W has some areas with no neighbours (one of the row sums equals zero).", call.=FALSE)    
-}
-
-
-#### Check and specify the priors
-    if(is.null(prior.mean.beta)) prior.mean.beta <- rep(0, p)
-    if(is.null(prior.var.beta)) prior.var.beta <- rep(1000, p)
-    if(is.null(prior.tau2)) prior.tau2 <- c(1, 0.01)
-    if(is.null(prior.nu2)) prior.nu2 <- c(1, 0.01)
-    if(is.null(prior.mean.alpha)) prior.mean.alpha <- rep(0, 1)
-    if(is.null(prior.var.alpha)) prior.var.alpha <- rep(1000, 1)
-    
-    if(length(prior.mean.beta)!=p) stop("the vector of prior means for beta is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.mean.beta)) stop("the vector of prior means for beta is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.mean.beta))!=0) stop("the vector of prior means for beta has missing values.", call.=FALSE)       
-    
-    if(length(prior.var.beta)!=p) stop("the vector of prior variances for beta is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.var.beta)) stop("the vector of prior variances for beta is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.var.beta))!=0) stop("the vector of prior variances for beta has missing values.", call.=FALSE)    
-    if(min(prior.var.beta) <=0) stop("the vector of prior variances for beta has elements less than zero", call.=FALSE)
-    
-    if(length(prior.tau2)!=2) stop("the prior value for tau2 is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.tau2)) stop("the prior value for tau2 is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.tau2))!=0) stop("the prior value for tau2 has missing values.", call.=FALSE)    
-    
-    if(length(prior.nu2)!=2) stop("the prior value for nu2 is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.nu2)) stop("the prior value for nu2 is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.nu2))!=0) stop("the prior value for nu2 has missing values.", call.=FALSE)   
-
-    if(length(prior.mean.alpha)!=1) stop("the prior mean for alpha is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.mean.alpha)) stop("the  prior mean for alpha is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.mean.alpha))!=0) stop("the prior mean for alpha has missing values.", call.=FALSE)       
-    if(length(prior.var.alpha)!=1) stop("the prior variance for alpha is the wrong length.", call.=FALSE)    
-    if(!is.numeric(prior.var.alpha)) stop("the  prior variance for alpha is not numeric.", call.=FALSE)    
-    if(sum(is.na(prior.var.alpha))!=0) stop("the  prior variance for alpha has missing values.", call.=FALSE)    
-    if(min(prior.var.alpha) <=0) stop("the prior variance for alpha has elements less than zero", call.=FALSE)
-    
-    
-    
-#### MCMC quantities
-## Checks
-    if(is.null(burnin)) stop("the burnin argument is missing", call.=FALSE)
-    if(is.null(n.sample)) stop("the n.sample argument is missing", call.=FALSE)
-    if(!is.numeric(burnin)) stop("burn-in is not a number", call.=FALSE)
-    if(!is.numeric(n.sample)) stop("n.sample is not a number", call.=FALSE) 
-    if(!is.numeric(thin)) stop("thin is not a number", call.=FALSE)
-    if(n.sample <= 0) stop("n.sample is less than or equal to zero.", call.=FALSE)
-    if(burnin < 0) stop("burn-in is less than zero.", call.=FALSE)
-    if(thin <= 0) stop("thin is less than or equal to zero.", call.=FALSE)
-    if(n.sample <= burnin)  stop("Burn-in is greater than n.sample.", call.=FALSE)
-    if(n.sample <= thin)  stop("thin is greater than n.sample.", call.=FALSE)
-    if(burnin!=round(burnin)) stop("burnin is not an integer.", call.=FALSE) 
-    if(n.sample!=round(n.sample)) stop("n.sample is not an integer.", call.=FALSE) 
-    if(thin!=round(thin)) stop("thin is not an integer.", call.=FALSE) 
-
-
-
-#### Set up matrices to store samples
+###############################    
+#### Set up the MCMC quantities    
+###############################
+#### Matrices to store samples
 n.keep <- floor((n.sample - burnin)/thin)
 samples.beta <- array(NA, c(n.keep, p))
 samples.alpha <- array(NA, c(n.keep, 1))
 samples.phi <- array(NA, c(n.keep, K))
 samples.delta <- array(NA, c(n.keep, K))
-if(!fix.rho.int) samples.rho <- array(NA, c(n.keep, 1))
-if(!fix.rho.slo) samples.lambda <- array(NA, c(n.keep, 1))
+    if(!fix.rho.int) samples.rho <- array(NA, c(n.keep, 1))
+    if(!fix.rho.slo) samples.lambda <- array(NA, c(n.keep, 1))
 samples.nu2 <- array(NA, c(n.keep, 1))
 samples.tau2 <- array(NA, c(n.keep, 2))
 colnames(samples.tau2) <- c("tau2.int", "tau2.slo")
 samples.fitted <- array(NA, c(n.keep, N.all))
 samples.like <- array(NA, c(n.keep, N.all))
 samples.deviance <- array(NA, c(n.keep, 1))
-if(n.miss>0) samples.Y <- array(NA, c(n.keep, n.miss))
+    if(n.miss>0) samples.Y <- array(NA, c(n.keep, n.miss))
 
     
-    
 #### Specify the Metropolis quantities
 accept.all <- rep(0,4)
 accept <- accept.all
@@ -243,55 +140,31 @@ tau2.phi.shape <- prior.tau2[1] + K/2
 tau2.delta.shape <- prior.tau2[1] + K/2
     
     
-    
-#### Spatial quantities
-## Create the triplet object
-W.triplet <- c(NA, NA, NA)
-    for(i in 1:K)
+
+##############################
+#### Specify spatial quantites
+##############################
+#### Create the determinant     
+    if(!fix.rho.int | !fix.rho.slo) 
     {
-        for(j in 1:K)
-        {
-            if(W[i,j]>0)
-            {
-                W.triplet <- rbind(W.triplet, c(i,j, W[i,j]))     
-            }else{}
-        }
-    }
-W.triplet <- W.triplet[-1, ]     
-W.n.triplet <- nrow(W.triplet) 
-W.triplet.sum <- tapply(W.triplet[ ,3], W.triplet[ ,1], sum)
-W.neighbours <- tapply(W.triplet[ ,3], W.triplet[ ,1], length)
-    
-    
-## Create the start and finish points for W updating
-W.begfin <- array(NA, c(K, 2))     
-temp <- 1
-    for(i in 1:K)
-    {
-        W.begfin[i, ] <- c(temp, (temp + W.neighbours[i]-1))
-        temp <- temp + W.neighbours[i]
-    }
-    
-    
-## Create the determinant     
-if(!fix.rho.int | !fix.rho.slo) 
-{
     Wstar <- diag(apply(W,1,sum)) - W
     Wstar.eigen <- eigen(Wstar)
     Wstar.val <- Wstar.eigen$values
-}else
-{}
+    }else
+    {}
+    if(!fix.rho.int) det.Q.rho <-  0.5 * sum(log((rho * Wstar.val + (1-rho))))    
+    if(!fix.rho.slo) det.Q.lambda <-  0.5 * sum(log((lambda * Wstar.val + (1-lambda))))     
 
-if(!fix.rho.int) det.Q.rho <-  0.5 * sum(log((rho * Wstar.val + (1-rho))))    
-if(!fix.rho.slo) det.Q.lambda <-  0.5 * sum(log((lambda * Wstar.val + (1-lambda))))     
 
     
-    
+##########################################
 #### Specify quantities that do not change
+########################################## 
 offset.mat <- matrix(offset, nrow=K, ncol=N, byrow=FALSE) 
 regression.mat <- matrix(X.standardised %*% beta, nrow=K, ncol=N, byrow=FALSE)   
 Y.mat <- matrix(Y, nrow=K, ncol=N, byrow=FALSE)
 Y.mat.miss <- matrix(Y.miss, nrow=K, ncol=N, byrow=FALSE)
+which.miss.mat <- matrix(which.miss, nrow=K, ncol=N, byrow=FALSE)
 phi.mat <- matrix(rep(phi, N), byrow=F, nrow=K)
 time.mat <- matrix(rep(time, K), byrow=TRUE, nrow=K)    
 delta.time.mat <- apply(time.mat, 2, "*", delta)
@@ -299,14 +172,15 @@ alpha.offset1 <- sum(time.mat^2 * which.miss.mat)
 ntime.miss <- apply(which.miss.mat,1,sum)
 ntime.miss2 <- apply(which.miss.mat * time.mat^2,1,sum)
 
-## Check for islands
+
+#### Check for islands
 W.list<- mat2listw(W)
 W.nb <- W.list$neighbours
 W.islands <- n.comp.nb(W.nb)
 islands <- W.islands$comp.id
 n.islands <- max(W.islands$nc)
-if(rho==1) tau2.phi.shape <- prior.tau2[1] + 0.5 * (K-n.islands)   
-if(lambda==1) tau2.delta.shape <- prior.tau2[1] + 0.5 * (K-n.islands)     
+    if(rho==1) tau2.phi.shape <- prior.tau2[1] + 0.5 * (K-n.islands)   
+    if(lambda==1) tau2.delta.shape <- prior.tau2[1] + 0.5 * (K-n.islands)     
 
 
 #### Beta update quantities
@@ -324,19 +198,19 @@ data.precision.beta <- t(X.short) %*% X.short
 ###########################
 #### Run the Bayesian model
 ###########################
-## Start timer
+#### Start timer
     if(verbose)
     {
-        cat("Generating", n.keep, "post burnin and thinned (if requested) samples\n", sep = " ")
-        
-        progressBar <- txtProgressBar(style = 3)
-        percentage.points<-round((1:100/100)*n.sample)
+    cat("Generating", n.keep, "post burnin and thinned (if requested) samples\n", sep = " ")
+    progressBar <- txtProgressBar(style = 3)
+    percentage.points<-round((1:100/100)*n.sample)
     }else
     {
-        percentage.points<-round((1:100/100)*n.sample)     
+    percentage.points<-round((1:100/100)*n.sample)     
     }
+
     
-    
+#### Create the MCMC samples    
     for(j in 1:n.sample)
     {
     ##################
@@ -380,18 +254,17 @@ data.precision.beta <- t(X.short) %*% X.short
     phi.offset2 <- apply(phi.offset,1, sum, na.rm=TRUE)
     temp1 <- gaussiancarupdate(W.triplet, W.begfin, W.triplet.sum, K, phi, tau2.phi, nu2, phi.offset2, rho, ntime.miss)
     phi <- temp1
-    if(rho<1)
-    {
+        if(rho<1)
+        {
         phi <- phi - mean(phi)
-    }else
-    {
+        }else
+        {
         phi[which(islands==1)] <- phi[which(islands==1)] - mean(phi[which(islands==1)])   
-    }
+        }
     phi.mat <- matrix(rep(phi, N), byrow=F, nrow=K)    
 
         
         
-        
     ####################
     ## Sample from delta
     ####################
@@ -401,13 +274,12 @@ data.precision.beta <- t(X.short) %*% X.short
     delta <- temp2
     if(lambda <1)
     {
-        delta <- delta - mean(delta)
+    delta <- delta - mean(delta)
     }else
     {
-        delta[which(islands==1)] <- delta[which(islands==1)] - mean(delta[which(islands==1)])   
+    delta[which(islands==1)] <- delta[which(islands==1)] - mean(delta[which(islands==1)])   
     }
     delta.time.mat <- apply(time.mat, 2, "*", delta)
-         
         
         
         
@@ -431,54 +303,52 @@ data.precision.beta <- t(X.short) %*% X.short
     ##################
     ## Sample from rho
     ##################
-    if(!fix.rho.int)
-    {
-    proposal.rho <- rtrunc(n=1, spec="norm", a=0, b=1, mean=rho, sd=proposal.sd.rho)   
-    temp3 <- quadform(W.triplet, W.triplet.sum, W.n.triplet, K, phi, phi, proposal.rho)
-    det.Q.proposal <- 0.5 * sum(log((proposal.rho * Wstar.val + (1-proposal.rho))))              
-    logprob.current <- det.Q.rho - temp2.phi / tau2.phi
-    logprob.proposal <- det.Q.proposal - temp3 / tau2.phi
-    prob <- exp(logprob.proposal - logprob.current)
+        if(!fix.rho.int)
+        {
+        proposal.rho <- rtruncnorm(n=1, a=0, b=1, mean=rho, sd=proposal.sd.rho)   
+        temp3 <- quadform(W.triplet, W.triplet.sum, W.n.triplet, K, phi, phi, proposal.rho)
+        det.Q.proposal <- 0.5 * sum(log((proposal.rho * Wstar.val + (1-proposal.rho))))              
+        logprob.current <- det.Q.rho - temp2.phi / tau2.phi
+        logprob.proposal <- det.Q.proposal - temp3 / tau2.phi
+        prob <- exp(logprob.proposal - logprob.current)
         
-    #### Accept or reject the proposal
-        if(prob > runif(1))
-        {
-        rho <- proposal.rho
-        det.Q.rho <- det.Q.proposal
-        accept[1] <- accept[1] + 1           
-        }else
-        {
-        }              
+        #### Accept or reject the proposal
+            if(prob > runif(1))
+            {
+            rho <- proposal.rho
+            det.Q.rho <- det.Q.proposal
+            accept[1] <- accept[1] + 1           
+            }else
+            {}              
         accept[2] <- accept[2] + 1           
-    }else
-    {}
+        }else
+        {}
         
      
        
     #####################
     ## Sample from lambda
     #####################
-    if(!fix.rho.slo)
-    {
-    proposal.lambda <- rtrunc(n=1, spec="norm", a=0, b=1, mean=lambda, sd=proposal.sd.lambda)   
-    temp3 <- quadform(W.triplet, W.triplet.sum, W.n.triplet, K, delta, delta, proposal.lambda)
-    det.Q.proposal <- 0.5 * sum(log((proposal.lambda * Wstar.val + (1-proposal.lambda))))              
-    logprob.current <- det.Q.lambda - temp2.delta / tau2.delta
-    logprob.proposal <- det.Q.proposal - temp3 / tau2.delta
-    prob <- exp(logprob.proposal - logprob.current)
+        if(!fix.rho.slo)
+        {
+        proposal.lambda <- rtruncnorm(n=1, a=0, b=1, mean=lambda, sd=proposal.sd.lambda)   
+        temp3 <- quadform(W.triplet, W.triplet.sum, W.n.triplet, K, delta, delta, proposal.lambda)
+        det.Q.proposal <- 0.5 * sum(log((proposal.lambda * Wstar.val + (1-proposal.lambda))))              
+        logprob.current <- det.Q.lambda - temp2.delta / tau2.delta
+        logprob.proposal <- det.Q.proposal - temp3 / tau2.delta
+        prob <- exp(logprob.proposal - logprob.current)
         
-    #### Accept or reject the proposal
-        if(prob > runif(1))
-        {
-        lambda <- proposal.lambda
-        det.Q.lambda <- det.Q.proposal
-        accept[3] <- accept[3] + 1           
-        }else
-        {
-        }              
+        #### Accept or reject the proposal
+            if(prob > runif(1))
+            {
+            lambda <- proposal.lambda
+            det.Q.lambda <- det.Q.proposal
+            accept[3] <- accept[3] + 1           
+            }else
+            {}              
         accept[4] <- accept[4] + 1           
-    }else
-    {}
+        }else
+        {}
         
      
        
@@ -497,24 +367,23 @@ data.precision.beta <- t(X.short) %*% X.short
     ###################
         if(j > burnin & (j-burnin)%%thin==0)
         {
-            ele <- (j - burnin) / thin
-            samples.beta[ele, ] <- beta
-            samples.phi[ele, ] <- phi
-            samples.delta[ele, ] <- delta
-            samples.alpha[ele, ] <- alpha
+        ele <- (j - burnin) / thin
+        samples.beta[ele, ] <- beta
+        samples.phi[ele, ] <- phi
+        samples.delta[ele, ] <- delta
+        samples.alpha[ele, ] <- alpha
             if(!fix.rho.int) samples.rho[ele, ] <- rho
             if(!fix.rho.slo) samples.lambda[ele, ] <- lambda
-            samples.nu2[ele, ] <- nu2
-            samples.tau2[ele, ] <- c(tau2.phi, tau2.delta)
-            samples.deviance[ele, ] <- deviance
-            samples.fitted[ele, ] <- fitted
-            samples.like[ele, ] <- like
+        samples.nu2[ele, ] <- nu2
+        samples.tau2[ele, ] <- c(tau2.phi, tau2.delta)
+        samples.deviance[ele, ] <- deviance
+        samples.fitted[ele, ] <- fitted
+        samples.like[ele, ] <- like
             if(n.miss>0) samples.Y[ele, ] <- rnorm(n=n.miss, mean=fitted[which.miss==0], sd=sqrt(nu2))
-            
         }else
-        {
-        }
+        {}
         
+    
         
     ########################################
     ## Self tune the acceptance probabilties
@@ -522,81 +391,58 @@ data.precision.beta <- t(X.short) %*% X.short
     k <- j/100
         if(ceiling(k)==floor(k))
         {
-        #### Determine the acceptance probabilities
-        accept.rho <- 100 * accept[1] / accept[2]
-        if(is.na(accept.rho)) accept.rho <- 45
-        accept.lambda <- 100 * accept[3] / accept[4]
-        if(is.na(accept.lambda)) accept.lambda <- 45
+            if(!fix.rho.int) proposal.sd.rho <- common.accceptrates2(accept[1:2], proposal.sd.rho, 40, 50, 0.5)
+            if(!fix.rho.slo) proposal.sd.lambda <- common.accceptrates2(accept[3:4], proposal.sd.lambda, 40, 50, 0.5)
         accept.all <- accept.all + accept
-        accept <- rep(0,4)
-            
-        #### rho tuning parameter
-        if(accept.rho > 50)
-        {
-            proposal.sd.rho <- min(proposal.sd.rho + 0.1 * proposal.sd.rho, 0.5)
-        }else if(accept.rho < 40)              
-        {
-            proposal.sd.rho <- proposal.sd.rho - 0.1 * proposal.sd.rho
+        accept <- rep(0,4)            
         }else
-        {
-        }
-        #### lambda tuning parameter
-        if(accept.lambda > 50)
-        {
-            proposal.sd.lambda <- min(proposal.sd.lambda + 0.1 * proposal.sd.lambda, 0.5)
-        }else if(accept.lambda < 40)              
-        {
-            proposal.sd.lambda <- proposal.sd.lambda - 0.1 * proposal.sd.lambda
-        }else
-        {
-        }    
-        }else
-        {   
-        }
+        {}
         
         
-        ################################       
-        ## print progress to the console
-        ################################
+    
+    ################################       
+    ## print progress to the console
+    ################################
         if(j %in% percentage.points & verbose)
         {
-            setTxtProgressBar(progressBar, j/n.sample)
+        setTxtProgressBar(progressBar, j/n.sample)
         }
     }
     
-    # end timer
+
+#### end timer
     if(verbose)
     {
-        cat("\nSummarising results")
-        close(progressBar)
+    cat("\nSummarising results")
+    close(progressBar)
     }else
     {}
     
     
+
 ###################################
 #### Summarise and save the results 
 ###################################
-## Compute the acceptance rates
-if(!fix.rho.int)
-{
+#### Compute the acceptance rates
+    if(!fix.rho.int)
+    {
     accept.rho <- 100 * accept.all[1] / accept.all[2]
-}else
-{
+    }else
+    {
     accept.rho <- NA    
-}
-
-if(!fix.rho.slo)
-{
+    }
+    if(!fix.rho.slo)
+    {
     accept.lambda <- 100 * accept.all[3] / accept.all[4]
-}else
-{
+    }else
+    {
     accept.lambda <- NA    
-}
+    }
 accept.final <- c(rep(100,4), accept.rho, accept.lambda)
 names(accept.final) <- c("beta", "alpha", "phi", "delta", "rho.int", "rho.slo")
     
       
-## Compute DIC
+#### Deviance information criterion (DIC)
 median.phi <- apply(samples.phi, 2, median)
 median.delta <- apply(samples.delta, 2, median)
 median.alpha <- median(samples.alpha)
@@ -612,14 +458,12 @@ p.d <- median(samples.deviance) - deviance.fitted
 DIC <- 2 * median(samples.deviance) - deviance.fitted    
 
 
-
 #### Watanabe-Akaike Information Criterion (WAIC)
 LPPD <- sum(log(apply(samples.like,2,mean)), na.rm=TRUE)
 p.w <- sum(apply(log(samples.like),2,var), na.rm=TRUE)
 WAIC <- -2 * (LPPD - p.w)
 
 
-
 #### Compute the Conditional Predictive Ordinate  
 CPO <- rep(NA, N.all)
     for(j in 1:N.all)
@@ -638,33 +482,8 @@ residuals <- data.frame(response=response.residuals, pearson=pearson.residuals,
     
     
 #### transform the parameters back to the origianl covariate scale.
-samples.beta.orig <- samples.beta
-number.cts <- sum(X.indicator==1)     
-    if(number.cts>0)
-    {
-        for(r in 1:p)
-        {
-            if(X.indicator[r]==1)
-            {
-                samples.beta.orig[ ,r] <- samples.beta[ ,r] / X.sd[r]
-            }else if(X.indicator[r]==2 & p>1)
-            {
-                X.transformed <- which(X.indicator==1)
-                samples.temp <- as.matrix(samples.beta[ ,X.transformed])
-                for(s in 1:length(X.transformed))
-                {
-                    samples.temp[ ,s] <- samples.temp[ ,s] * X.mean[X.transformed[s]]  / X.sd[X.transformed[s]]
-                }
-                intercept.adjustment <- apply(samples.temp, 1,sum) 
-                samples.beta.orig[ ,r] <- samples.beta[ ,r] - intercept.adjustment
-            }else
-            {
-            }
-        }
-    }else
-    {
-    }
-    
+samples.beta.orig <- common.betatransform(samples.beta, X.indicator, X.mean, X.sd, p, FALSE)
+
     
 #### Create a summary object
 samples.beta.orig <- mcmc(samples.beta.orig)
@@ -684,69 +503,68 @@ summary.hyper[1, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.alpha)), gewe
 summary.hyper[2, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.tau2[ ,1])), geweke.diag(mcmc(samples.tau2[ ,1]))$z)   
 summary.hyper[3, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.tau2[ ,2])), geweke.diag(mcmc(samples.tau2[ ,2]))$z)   
 summary.hyper[4, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.nu2)), geweke.diag(mcmc(samples.nu2))$z)   
-
-if(!fix.rho.int)
-{
+    if(!fix.rho.int)
+    {
     summary.hyper[5, 1:3] <- quantile(samples.rho, c(0.5, 0.025, 0.975))
     summary.hyper[5, 4:7] <- c(n.keep, accept.rho, effectiveSize(samples.rho), geweke.diag(samples.rho)$z)
-}else
-{
+    }else
+    {
     summary.hyper[5, 1:3] <- c(rho, rho, rho)
     summary.hyper[5, 4:7] <- rep(NA, 4)
-}
-
-if(!fix.rho.slo)
-{
+    }
+    if(!fix.rho.slo)
+    {
     summary.hyper[6, 1:3] <- quantile(samples.lambda, c(0.5, 0.025, 0.975))
     summary.hyper[6, 4:7] <- c(n.keep, accept.lambda, effectiveSize(samples.lambda), geweke.diag(samples.lambda)$z)
-}else
-{
+    }else
+    {
     summary.hyper[6, 1:3] <- c(lambda, lambda, lambda)
     summary.hyper[6, 4:7] <- rep(NA, 4)
-}   
-
-
+    }   
 summary.results <- rbind(summary.beta, summary.hyper)
 summary.results[ , 1:3] <- round(summary.results[ , 1:3], 4)
 summary.results[ , 4:7] <- round(summary.results[ , 4:7], 1)
     
     
-## Compile and return the results
+#### Compile and return the results
 loglike <- (-0.5 * deviance.fitted)
 modelfit <- c(DIC, p.d, WAIC, p.w, LMPL, loglike)
 names(modelfit) <- c("DIC", "p.d", "WAIC", "p.w", "LMPL", "loglikelihood")
 
 
-if(fix.rho.int & fix.rho.slo)
-{
+#### Harmonise samples in case of them not being generated
+    if(fix.rho.int & fix.rho.slo)
+    {
     samples.rhoext <- NA
-}else if(fix.rho.int & !fix.rho.slo)
-{
+    }else if(fix.rho.int & !fix.rho.slo)
+    {
     samples.rhoext <- samples.lambda
     names(samples.rhoext) <- "rho.slo"
-}else if(!fix.rho.int & fix.rho.slo)
-{
+    }else if(!fix.rho.int & fix.rho.slo)
+    {
     samples.rhoext <- samples.rho  
     names(samples.rhoext) <- "rho.int"
-}else
-{
+    }else
+    {
     samples.rhoext <- cbind(samples.rho, samples.lambda)
     colnames(samples.rhoext) <- c("rho.int", "rho.slo")
-}
-
-if(n.miss==0) samples.Y = NA
+    }
+    if(n.miss==0) samples.Y = NA
 
 samples <- list(beta=mcmc(samples.beta.orig), alpha=mcmc(samples.alpha), phi=mcmc(samples.phi),  delta=mcmc(samples.delta), tau2=mcmc(samples.tau2), nu2=mcmc(samples.nu2), rho=mcmc(samples.rhoext), fitted=mcmc(samples.fitted), Y=mcmc(samples.Y))        
 model.string <- c("Likelihood model - Gaussian (identity link function)", "\nLatent structure model - Spatially autocorrelated linear time trends\n")
 results <- list(summary.results=summary.results, samples=samples, fitted.values=fitted.values, residuals=residuals, modelfit=modelfit, accept=accept.final, localised.structure=NULL, formula=formula, model=model.string,  X=X)
 class(results) <- "CARBayesST"
+
+
+#### Finish by stating the time taken 
     if(verbose)
     {
-        b<-proc.time()
-        cat(" finished in ", round(b[3]-a[3], 1), "seconds")
+    b<-proc.time()
+    cat(" finished in ", round(b[3]-a[3], 1), "seconds")
     }else
     {}
-    return(results)
+return(results)
 }
 
 
